/*==============================================================================

      Sons de animais
      
      Autor: Claudio Lrios
      
      
      Data:  10/09/2014
      
      Objetivo: Reproduzir sons gravados em eeprom externa 24c256
      Ao apertar um interruptor obtemos um som de animal, sendo 7
      diferentes.
      
      Sons gerados ao apertar os interruptores:
       
      pino  4= chipanz/orangotango
      pino  5= carneiro 
      pino  6= elefante
      pino  7= vaca
      pino  8= cavalo
      pino  9= gato
      pino 10= cachorro
      
      Obs. A memria EEprom 24c256 dever ser gravada com arquivo fornecido.
      Poder usar o programa PonyProg ou outro equivalente.
      
      Utiliza a tcnica de reproduo com apenas 1 bit de saida.
      
      Use PIC16f676 com as seguintes ligaes em seus pinos:
      
      1- vcc (+5v)
      2- xtal 20 mhz
      3- xtal 
      4- interuptor ao gnd com pullup de 4k7 (ao vcc)
      5- interuptor ao gnd com pullup de 4k7 (ao vcc)
      6- interuptor ao gnd com pullup de 4k7 (ao vcc)
      7- interuptor ao gnd com pullup de 4k7 (ao vcc)
      8- interuptor ao gnd com pullup de 4k7 (ao vcc)
      9- interuptor ao gnd com pullup de 4k7 (ao vcc)
     10- interuptor ao gnd com pullup de 4k7 (ao vcc)
     11- Sda (dado)  da EEprom
     12- Scl (clock) da EEprom
     13- Sada de som (filtre com resistor 10k + capacitor 0,22 ao gnd
     14- gnd
     
     Use compilador CCS C  ou similar.

     Este arquivo  parte integrante do blog 'Larios.Tecnologia.Ws'


==============================================================================*/
#include <16F676.h>
#device adc=8
#use delay(clock=20000000)// use cristal externo de 20 MHZ
#use fast_io(a)
#zero_ram
#fuses NOWDT,HS , NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT

#byte   trisa = 0x85
#byte   porta = 0x05
#byte   trisc = 0x87
#byte   portc = 0x07
#bit    sda  = 0x05.2  //pino 11
#bit    scl  = 0x05.1  //pino 12
#bit    som  = 0x05.0  //pino 13
#bit    f0   = 0x07.0  //pino 10
#bit    f1   = 0x07.1  //pino 9
#bit    f2   = 0x07.2  //pino 8
#bit    f3   = 0x07.3  //pino 7
#bit    f4   = 0x07.4  //pino 6
#bit    f5   = 0x07.5  //pino 5
#bit    f6   = 0x05.3  //pino 4

#bit    tris_sda  = 0x85.2  //sentido do pino 5

//constantes
int8 const  tcl  =  5;
int8 const  vel_reproducao = 90 ;//velocidade da reproduo (tentativa e erro)


/*==============================================================================
       SUB-ROTINAS   I2C
==============================================================================*/
void start_i2c(){ //START
  tris_sda=1; //alta impedancia
  scl=1; //
  delay_us(tcl);
  sda=0;
  tris_sda=0; //saida zerada
  delay_us(tcl);
  scl=0;  //zera scl
  delay_us(tcl);
  tris_sda=1; //alta impedancia
}

void stop_i2c(){ //STOP
  sda=0;
  tris_sda=0;
  scl=0;
  delay_us(tcl);
  scl=1;
  delay_us(tcl);
  tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)
}

void ack(){ //ACKNOLEDGE
   sda=0;
   tris_sda=0;
   scl=1;
   delay_us(tcl);
   scl=0;
   delay_us(tcl);
   tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)

}
void no_ack(){ //NO ACKNOLEDGE
  
   tris_sda=1; //sda =1 (por alta impedancia + resistor pullup)
   scl=1;
   delay_us(tcl);
   scl=0;
   delay_us(tcl);
  }
  
void send_i2c(int b){ //ENVIA BYTE PARA EEPROM 24C256
int a;
  tris_sda=0;
  for(a=0;a<8;a++){
  if(b&0x80){ sda=1;} else {sda=0;}
  scl=1;
  delay_us(tcl);
  scl=0;
  b=b<<1;
  }

}


void reproduzir(int16 init, int16 end){// REPRODUO DO INTERVALO INFORMADO
 int16 tam;
 int8  d,f;

  for(f=0;f<3;f++){  //certifica que realmente houve troca endereo de acesso
   start_i2c(); //inicia condio de start
   delay_us(50);
   send_i2c(0xa0);//device write
   ack();
   send_i2c(make8(init,1));// endereo alto a ser reproduzido
   ack();
   send_i2c(make8(init,0));//endereo baixo a ser reproduzido
   no_ack();
   delay_us(50);
  }
   
   start_i2c(); //inicia condio de start  
   send_i2c(0xa1);//device read
   ack();
   tris_sda=1; //entrada

   for(tam=init;tam<end;tam++){//quantidade a ser reproduzida

    for(d=0;d<8;d++){
    if(sda) {som=1;} else {som=0;}
    scl=1;
    delay_us(tcl);
    scl=0;
    delay_us(vel_reproducao);   //valor para ajuste da velocidade de reproduo
    }
    ack();

   }

   no_ack();
   stop_i2c();//stop
   delay_us(50);
}


/*==============================================================================
             MAIN
==============================================================================*/



void main() {

   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_OFF);
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   porta=0;
   portc=0;
   trisa= 0b111100;
   trisc= 0b111111;

/*==============================================================================
             LOOP  PRINCIPAL
==============================================================================*/



   while(1){

   if(!f0){  reproduzir(0x00,7682/8); delay_ms(500);   } //cachorro

   if(!f1){ reproduzir(7684/8,32482/8); delay_ms(500); } //gato

   if(!f2){reproduzir(32484/8,53442/8); delay_ms(500); } //cavalo

   if(!f3){reproduzir(53444/8,99586/8); delay_ms(500); }  //vaca

   if(!f4){ reproduzir(99588/8,130690/8); delay_ms(500);} //elefante

   if(!f5){reproduzir(130692/8,135170/8); delay_ms(500);} //carneiro

   if(!f6){reproduzir(135172/8,234440/8); delay_ms(500);} //chipanze/orangotango

   

   }//while


}//main
